#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "_lock4xi.py">
#<abstract4xi abstract = "providing services for the object _lock4xi"/>
#<refer4xi reference = "reduction.py"/>  
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ace_lock4xi',
            'reason_lock4xi',            
            'sense_lock4xi',
            'reset_lock4xi',
            'release_lock4xi',            
            'acquire_lock4xi',
            'acquire_lock4xi',
            'get_lock4xi',            
            'init_lock4xi'] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
import sys
import threading                
import _thread
import _winapi   

import socket
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from print4xi import print4xi
from time4xi import time4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global _lock4xi
#"
#/>
 
    
# -------------------------------------------------------------------------------------------------
#<function4xi function = "init-lock4xi" usage = "
#   initializes lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def init_lock4xi(): 
    global _lock4xi
    
    '''
    initializes lock
    '''
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
       
    class2print = "init_lock4xi"
    method2print = "init_lock4xi"
    
    message2print = "{started" + "; init-lock4xi-started: " + str(init_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
    ack2lock = False
    nack2lock = True
    
    try:
       _lock4xi = threading.Lock()    

       message2print = "{initialized" + "; lock4xi-initialized-by-init-lock4xi to: " + str(_lock4xi)  + "; time: " + time4xi() + "}"
       print4xi(class2print, method2print, message2print) 
       
       ack2lock = True   
       
       message2print = "{set" + "; ack2lock-set-by-lock4xi to: " + str(ack2lock)  + "; time: " + time4xi() + "}"
       print4xi(class2print, method2print, message2print)          
                           
       
    except:
       message2print = "{yielded" + "; nack2lock-yielded-by-lock4xi: " + str(nack2lock)  + "; time: " + time4xi() + "}"
       print4xi(class2print, method2print, message2print) 

            
       
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------                   
    message2print = "{returning" + "; init-lock4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                           
#</function4xi>
           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "get-lock4xi" usage = "
#   gets lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def get_lock4xi():
    global _lock4xi
    
    '''
    gets lock
    '''
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   

    class2print = "get_lock4xi"
    method2print = "get_lock4xi"
    
    message2print = "{started" + "; get-lock4xi-started: " + str(get_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # ----------------------------------------------------------------------------------------- 
           
    if _lock4xi is None:
        init_lock4xi() 
        
        message2print = "{performed" + "; init-lock4xi-performed-by-get-lock4xi: " + str(init_lock4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                       
    
    # -----------------------------------------------------------------------------------------    
    #
    # ----------------------------------------------------------------------------------------- 
            
    message2print = "{returning" + "; lock4xi-returning-by-get-lock4xi: " + str(_lock4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)   
            
            
    return _lock4xi
#</function4xi>
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "acquire-lock4xi" usage = "
# 1) _get_listener4xi.py <-- compatible and familiar look
#         import _lock4xi
#         _lock4xi.acquire()
# 2) from _lock4xi import acquire_lock4xi
#         acquire_lock4xi()   
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def acquire_lock4xi():
    global _lock4xi

    
    '''
    acquires lock
    '''
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   
    class2print = "acquire_lock4xi"
    method2print = "acquire_lock4xi"
    
    message2print = "{started" + "; acquire-lock4xi-started: " + str(acquire_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
    if _lock4xi is None:
        init_lock4xi()
        
        message2print = "{performed" + "; init-lock4xi-performed-by-acquire-lock4xi: " + str(init_lock4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                     
    
    _lock4xi.acquire() 
    
    message2print = "{performed" + "; lock4xi-acquire-performed-by-acquire-lock4xi: " + str(_lock4xi.acquire) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)             
               
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------                   
    message2print = "{returning" + "; acquire-lock4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                  
#</function4xi>
        
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release-lock4xi" usage = "
# # 1) _get_listener4xi.py <-- compatible and familiar look
#         import _lock4xi
#             _lock4xi.acquire()
#             ....
#         _   lock4xi.release()
# 2) from _lock4xi import acquire_lock4xi <-- use function instead of global variable
#    from _lock4xi import release_lock4xi
#        acquire_lock4xi()
#        ......
#        release_lock4xi() 
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def release_lock4xi(): 
    global _lock4xi
    
    '''
    releases lock
    '''
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   

    class2print = "release_lock4xi"
    method2print = "release_lock4xi"
    
    message2print = "{started" + "; release-lock4xi-started: " + str(release_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
    if _lock4xi is None:
        init_lock4xi()
             
        message2print = "{performed" + "; init-lock4xi-performed-by-release-lock4xi: " + str(init_lock4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
           
    _lock4xi.release()  
    
    
    message2print = "{performed" + "; lock4xi-release-performed-by-release-lock4xi: " + str(_lock4xi.release) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)             
            
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------                   
    message2print = "{returning" + "; release-lock4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                
#</function4xi>
        
            
# -------------------------------------------------------------------------------------------------
# reset_lock4xi
#<function4xi function = "reset-lock4xi" usage = "
#   resets lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def reset_lock4xi():
    global _lock4xi

    
    '''
    resets lock
    '''
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   
    class2print = "reset_lock4xi"
    method2print = "reset_lock4xi"
    
    message2print = "{started" + "; reset-lock4xi-started: " + str(reset_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        

    _lock4xi = threading.Lock()     

    message2print = "{reset" + "; _lock4xi-reset-by-reset-lock4xi: " + str(_lock4xi) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------                   
    message2print = "{returning" + "; reset-lock4xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
                 
#</function4xi>
    
   
# -------------------------------------------------------------------------------------------------
#<function4xi function = "sense-lock4xi" usage = "
#   senses lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def sense_lock4xi():
    global _lock4xi
    
    '''
    senses lock
    '''
        
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   

    class2print = "sense_lock4xi"
    method2print = "sense_lock4xi"
    
    message2print = "{started" + "; sense-lock4xi-started: " + str(sense_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
            
    message2print = "{sensing" + "; lock4xi-sensing-by-sense-lock4xi: " + str(_lock4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)   
            
            
    return _lock4xi    
#</function4xi>
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "reason-lock4x" usage = "
#   reasons lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def reason_lock4xi(): 
    global _lock4xi
    
    '''
    reasons lock
    '''
    
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   
    class2print = "reason_lock4xi"
    method2print = "reason_lock4xi"
    
    message2print = "{started" + "; reason-lock4xi-started: " + str(reason_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------               
    if _lock4xi is None:
    
        message2print = "{reasoning" + "; lock4xi-reasoning-by-reason-lock4xi: ** no thread going on - _lock4xi = " + str(None) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)       
    else: 
        message2print = "{reasoning" + "; lock4xi-reasoning-by-reason-lock4xi: a thread running - _lock4xi = " + str(_lock4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
            
            
    return _lock4xi
#</function4xi>
     
# -------------------------------------------------------------------------------------------------
#<function4xi function = "ace-lock4xi" usage = "
#   aces lock
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
def ace_lock4xi():
    global _lock4xi

    '''
    aces lock
    '''

    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
   
    class2print = "ace_lock4xi"
    method2print = "ace_lock4xi"
    
    message2print = "{started" + "; ace-lock4xi-started: " + str(ace_lock4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    # -----------------------------------------------------------------------------------------    
    #
    # -----------------------------------------------------------------------------------------        
    if _lock4xi is None:
    
        message2print = "{acing" + "; lock4xi-acing-by-ace-lock4xi: ** nothing to be aced - _lock4xi = " + str(None) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)       
    else: 
        message2print = "{acing" + "; lock4xi-acing-by-ace-lock4xi: a thread running - _lock4xi = " + str(_lock4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
            
            
    return _lock4xi       
#</function4xi>
#</program4xi>
# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_lock4xi.txt
#   sense-test-lock4xi.xml
#   reason-test-lock4xi.xml
#   ace-test-lock4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of _lock4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
